% Copyright 2018. All Rights Reserved
% Code by Shuang Li
% For Paper, "An Optimization View of MUSIC and Its Extension to Missing Data"
% arXiv:1806.03511, 2018
% by Shuang Li, Hassan Mansour, and Michael B. Wakin
% This code is used to reproduce Figure 9.


%%
clc;clear; close all;

trial=500;                % number of trials
M=50;                     % number of time samples
r = [1 1];                % true damping ratios
f1 = 0.1;
delta_f = 1/M;            % separation between true frequencies
f = [f1 f1+delta_f];      % true frequencies 
K = 2;                    % number of frequencies
Mp = 40;                  % number of measurements per sensor
N=10;                     % number of sensors
Time=0:M-1;
Time=Time';
spike = 1:0.5:10;         % the first entry in Phi
c = [1 1];                % coefficients
count_data=zeros(1,length(spike));
coherence_data=zeros(trial,length(spike));

for t=1:trial
    indx_me=randperm(N*M,Mp*N);  % observation index
    for ff=1:length(spike)
        fprintf('Trial=%d,spike=%.1f\n',t,spike(ff))
        
        Phi = dftmtx(N);       
        Phi = Phi(:,1:K);
        Phi(1,1) = spike(ff);
        Phi = Phi*diag(1./norms(Phi));
                      
        Exp = exp(1j*2*pi*Time*f);
        Arf = Exp;
        chat = c.*norms(Arf);
        Arf = Arf*diag(1./norms(Arf));
        
        Data = Arf*diag(chat)*Phi';     % true data matrix
        [UD, SD, VD] = svd(Data);
        UD = UD(:,1:K);
        VD = VD(:,1:K);
        %mu1 = max(norms(UD').^2)*size(UD,1)/K;
        mu2 = max(norms(VD').^2)*size(VD,1)/K;
        coherence_data(t,ff) = mu2;
               
        CData=zeros(M,N);
        CData(indx_me)=Data(indx_me);        % data matrix with random missing entries
        
        % Parameters estimation via MD-MUSIC
        cvx_begin quiet
        variable T(M,M) hermitian;
        variable Xn(M,N) complex;
        variable W(N,N) Hermitian;
        dual variable Y0
        minimize(0.5*trace(T)+0.5*trace(W))
        subject to
        Y0:[T   Xn; ...
            Xn'   W] == semidefinite(M+N,M+N);
        Xn(indx_me) == CData(indx_me);
        cvx_end
        
        
        if norm(Xn-Data,'fro')/norm(Data,'fro') <= 1e-5
            count_data(ff)=count_data(ff)+1;   % count successful probability
        end
        
    end
    count_data
    
end
if trial == 1
    co_data = coherence_data;
else
    co_data = mean(coherence_data);
end
p_data=count_data./trial;   % probability of successfull recovery for data


%% ==================== Show Results =======================
figure;
fs = 22;
lw = 3;
plot(spike,co_data,'b-o','LineWidth',lw);
xlabel('$\phi_{1,1}$','Interpreter','LaTex','FontSize',fs);
ylabel('$\mu_0^\star$','Interpreter','LaTex','FontSize',fs);
set(gca,'FontSize',fs);
axis tight;


figure;
plot(spike,p_data,'b-o','LineWidth',lw);
xlabel('$\phi_{1,1}$','Interpreter','LaTex','FontSize',fs);
ylabel('Probability of successful recovery','FontSize',fs-1);
set(gca,'FontSize',fs);
axis tight;





